<?php
namespace ChilexpressOficial\Chilexpress\Model\Carrier;


use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Helper;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;

class Chilexpress extends AbstractCarrier implements CarrierInterface
{
    /**
     * Carrier helper
     *
     * @var Helper\Carrier
     */
    protected $_carrierHelper;

    /**
     * @var \ChilexpressOficial\Chilexpress\Helper\Data
     */
    protected $_helperData;

    /**
    * @var string
    */
    protected $_code = 'chilexpress';

    /**
     * @var bool
     */
    protected $_isFixed = false;

    /**
     * @var ResultFactory
     */
    protected $rateResultFactory;

    /**
     * @var MethodFactory
     */
    protected $rateMethodFactory;

    /**
     * @var \Magento\Shipping\Model\Tracking\Result\StatusFactory
     */

    protected $_trackStatusFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    protected $customerSession;

    protected $serviceTypeCode = -1;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param StatusFactory $trackStatusFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
     * @param Helper\Carrier $carrierHelper
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Magento\Customer\Model\Session $customerSession,
        \ChilexpressOficial\Chilexpress\Helper\Data $helperData,
        \Magento\Shipping\Model\Tracking\Result\StatusFactory $trackStatusFactory,
        Helper\Carrier $carrierHelper,
        array $data = []
    ) {
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
        $this->trackStatusFactory = $trackStatusFactory;
        $this->customerSession = $customerSession;
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        $this->_helperData = $helperData;
        $this->_trackStatusFactory = $trackStatusFactory;
        $this->_carrierHelper = $carrierHelper;
    }

    /**
     * @param RateRequest $request
     * @return bool|Result
     */
    public function collectRates(RateRequest $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }
        if ($request->getDestCountryId() != 'CL') {
            return false;
        }

        /*********************************************/

        $packageWeight = $request->getPackageWeight();
        if ( !$packageWeight) {
            $packageWeight = 1;
        }

        $length  = $request->getPackageDepth();
        $width  = $request->getPackageWidth();
        $height  = $request->getPackageHeight();

        if (!$height) {
            $height = 1;
        }

        if (!$length) {
            $length = 1;
        }

        if (!$width) {
            $width = 1;
        }
        
        // echo "weight($packageWeight) height($height) length($length) width($width)";

        $json_response = $this->_helperData->getCotizacion(
                        $request->getDestCity(),
                        $this->_helperData->getOrigenConfig('comuna_origen'),
                        $packageWeight, $height,$width, $length, 100);

        //print_r($json_response);
        //die();

        if (!$json_response || $json_response->statusCode != 0) {
            return false; // Algo fallo al obtener la cotizacion
        }
        $precio_chilexpress = 0;
        $opciones_de_envio = $json_response->data->courierServiceOptions;
        foreach($opciones_de_envio as $opcion) {
            if($opcion->serviceTypeCode == $this->serviceTypeCode)
            {
                $precio_chilexpress = intval($opcion->serviceValue);
            }
        }
        
        if ($precio_chilexpress == 0) {
            return false; // No hay ningun tipo de envio válido disponible
        }

        /*********************************************/
        /** @var \Magento\Shipping\Model\Rate\Result $result */
        $result = $this->rateResultFactory->create();
       
        /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
        $method = $this->rateMethodFactory->create();

        $method->setCarrier($this->_code);
        $method->setCarrierTitle($this->getConfigData('title'));

        $method->setMethod($this->_code);
        $method->setMethodTitle($this->getConfigData('name'));

        $shippingCost = $precio_chilexpress;

        $method->setPrice($shippingCost);
        $method->setCost($shippingCost);
        $result->append($method);

        return $result;
    }

    /**
     * @return array
     */
    public function getAllowedMethods()
    {
        return [$this->_code => $this->getConfigData('name')];
    }

    /**
     * Check if carrier has shipping tracking option available
     *
     * @return bool
     */
    public function isTrackingAvailable()
    {
        return true;
    }


     /**
     * Check if carrier has shipping label option available
     *
     * @return bool
     */
    public function isShippingLabelsAvailable()
    {
        return true;
    }

    /**
     * Is state province required
     *
     * @return bool
     */
    public function isStateProvinceRequired()
    {
        return true;
    }

    /**
     * Check if city option required
     *
     * @return bool
     */
    public function isCityRequired()
    {
        return true;
    }

    /**
     * Determine whether zip-code is required for the country of destination
     *
     * @param string|null $countryId
     * @return bool
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function isZipCodeRequired($countryId = null)
    {
        return false;
    }

    /**
     * Do shipment request to carrier web service, obtain Print Shipping Labels and process errors in response
     *
     * @param \Magento\Framework\DataObject $request
     * @return \Magento\Framework\DataObject
     */
    protected function _doShipmentRequest(\Magento\Framework\DataObject $request)
    {
         
    }

    /**
     * Do request to shipment
     *
     * @param \Magento\Shipping\Model\Shipment\Request $request
     * @return array|\Magento\Framework\DataObject
     */
    public function requestToShipment($request)
    {
        $packages = $request->getPackages();
        if (!is_array($packages) || !$packages) {
            throw new \Magento\Framework\Exception\LocalizedException(__('No packages for request'));
        }

        $request->setOrigCountryId('CL');
        /************************/
        $customsValue = 0;
        $packageWeight = 0;
        $packages = $request->getPackages();
        $paquetes = [];
        $shipment_id = $request->getOrderShipment()->getOrder()->getId();
        $order_id = $request->getOrderShipment()->getOrder()->getId();
        $index = 1;
        foreach ($packages as &$piece) {
            $params = $piece['params'];
            // TODO: Convertir a centimetros y KG
            $packageWeight = $piece['params']['weight'];

            if ($packageWeight != \Zend_Measure_Weight::KILOGRAM) {
                $packageWeight = round(100*(
                    $this->_carrierHelper->convertMeasureWeight(
                        (float)$piece['params']['weight'],
                        $piece['params']['weight_units'],
                        \Zend_Measure_Weight::KILOGRAM
                    )
                ))/100;
                $piece['params']['weight_units'] = \Zend_Measure_Weight::KILOGRAM;
            }

            $length = $piece['params']['length'];
            $width = $piece['params']['width'];
            $height =  $piece['params']['height'];
            if ($piece['params']['dimension_units'] != \Zend_Measure_Length::CENTIMETER) {
                $length = round(
                    $this->_carrierHelper->convertMeasureDimension(
                        (float)$piece['params']['length'],
                        $piece['params']['dimension_units'],
                        \Zend_Measure_Length::CENTIMETER
                    )
                );
                $width = round(
                    $this->_carrierHelper->convertMeasureDimension(
                        (float)$piece['params']['width'],
                        $piece['params']['dimension_units'],
                        \Zend_Measure_Length::CENTIMETER
                    )
                );
                $height = round(
                    $this->_carrierHelper->convertMeasureDimension(
                        (float)$piece['params']['height'],
                        $piece['params']['dimension_units'],
                        \Zend_Measure_Length::CENTIMETER
                    )
                );
                $piece['params']['dimension_units'] = \Zend_Measure_Length::CENTIMETER;
            }

            $piece['params']['height'] = $height;
            $piece['params']['length'] = $length;
            $piece['params']['width'] = $width;

            $piece['params']['dimension_units'] = 'CENTIMETER';
            $piece['params']['weight'] = $packageWeight;
            $piece['params']['weight_units'] = 'KILOGRAM';
            
            $paquetes[] =  array(
                "weight"=> $packageWeight, // Peso en kilogramos
                "height"=> $height, // Altura en centímetros
                "width"=> $width, // Ancho en centímetros
                "length"=> $length,  // Largo en centímetros
                "serviceDeliveryCode"=> 3, // Código del servicio de entrega, obtenido de la API Cotización
                "productCode"=> "3", // Código del tipo de roducto a enviar; 1 = Documento, 3 = Encomienda
                "deliveryReference"=> "ORDEN-".$order_id, // Referencia que permite identificar el envío por parte del cliente.
                "groupReference"=> "ORDEN-".$order_id."-GRUPO-".$index, // Referencia que permite identificar un grupo de bultos que va por parte del cliente.
                "declaredValue"=> $piece['params']['customs_value'], // Valor declarado del producto
                "declaredContent"=> "string", // Tipo de producto enviado; 1 = Moda, 2 = Tecnologia, 3 = Repuestos, 4 = Productos medicos, 5 = Otros
                "extendedCoverageAreaIndicator"=> false, // Indicador de contratación de cobertura extendida 0 = No, 1 = Si
                "receivableAmountInDelivery"=> 1000 // Monto a cobrar, en caso que el cliente tenga habilitada esta opción.
              );

            $customsValue += $piece['params']['customs_value'];
            $packageWeight += $piece['params']['weight'];
            $index++;
        }

        $request->setPackages($packages)
                ->setPackageWeight($packageWeight)
                ->setPackageValue($customsValue)
                ->setValueWithDiscount($customsValue)
                ->setPackageCustomsValue($customsValue)
                ->setFreeMethodWeight(0);

        /********/

        $payload_header = array(
                "certificateNumber" => 0, //Número de certificado, si no se ingresa se creará uno nuevo
                "customerCardNumber"=> $this->_helperData->getOrigenConfig('numero_tcc_origen'), // Número de Tarjeta Cliente Chilexpress (TCC)
                "countyOfOriginCoverageCode"=> $this->_helperData->getOrigenConfig('comuna_origen'), // Comuna de origen
                "labelType"=> 2, // Imagen
                "marketplaceRut"=> $this->_helperData->getRemitenteConfig("rut_marketplace_remitente"), // Rut asociado al Marketplace
                "sellerRut"=> "DEFAULT" // Rut asociado al Vendedor
            );

        $payload_address_devolucion = array(
                "addressId"=> 0,
                "countyCoverageCode"=> $this->_helperData->getDevolucionConfig('comuna_devolucion'),
                "streetName"=> $this->_helperData->getDevolucionConfig('calle_devolucion'),
                "streetNumber"=> $this->_helperData->getDevolucionConfig('numero_calle_devolucion'),
                "supplement"=> $this->_helperData->getDevolucionConfig('complemento_devolucion'),
                "addressType"=> "DEV",
                "deliveryOnCommercialOffice"=> false,
                "observation"=> "DEFAULT"
            );

        $payload_contact_devolucion = array(
                "name"=> $this->_helperData->getRemitenteConfig('nombre_remitente'),
                "phoneNumber"=> $this->_helperData->getRemitenteConfig('telefono_remitente'),
                "mail"=> $this->_helperData->getRemitenteConfig('email_remitente'),
                "contactType"=> "R" // Tipo de contacto; Destinatario (D), Remitente (R)
            );
        $streetLine3 = $request->getOrderShipment()->getShippingAddress()->getStreetLine(3);
        $payload_address_destino = [
                "addressId" => 0,
                "countyCoverageCode"=> $request->getRecipientAddressCity(), // Cobertura de destino obtenido por la API Consultar Coberturas
                "streetName"=> $request->getRecipientAddressStreet1(), // Nombre de la calle
                "streetNumber"=> $request->getRecipientAddressStreet2(), // Numeración de la calle
                "supplement"=> $streetLine3, // Información complementaria de la dirección
                "addressType"=> "DEST", // Tipo de dirección; DEST = Entrega, DEV = Devolución.
                "deliveryOnCommercialOffice"=> false, // Indicador si es una entrega en oficina comercial (true) o entrega en domicilio (false)
                "commercialOfficeId"=> "",
                "observation"=> "DEFAULT" // Observaciones adicionales
            ];

        $payload_contact_destino = array(
                "name"=> $request->getRecipientContactPersonName(),
                "phoneNumber"=> $request->getRecipientContactPhoneNumber(),
                "mail"=> $request->getRecipientEmail(),
                "contactType"=> "D" // Tipo de contacto; Destinatario (D), Remitente (R)
            );

            $payload = [
                "header" => $payload_header,
                "details" => [
                      [
                          "addresses" => array(
                              $payload_address_destino,
                              $payload_address_devolucion
                          ),
                          "contacts" => array( // Se debe entregar un detalle para los datos de contacto del destinatario (D) y otro para los del remitente (R)
                              $payload_contact_devolucion,
                              $payload_contact_destino
                          ),
                          "packages" => $paquetes
                      ]
                ]
            ];
        
        // echo json_encode($payload);
        
        $json_response = $this->_helperData->generarOT($payload);
        $data = [];
        if ($json_response->statusCode != 0) {
            // TODO better show the error
            return null;
        }

        /************************/
        foreach($json_response->data->detail as $detail)
        {
            $data[] = [
                'tracking_number' => $detail->transportOrderNumber,
                'label_content' => base64_decode($detail->label->labelData),
            ];
        }

        $response = new \Magento\Framework\DataObject(['info' => $data]);
        if(count($data) > 0){
            $request->setMasterTrackingId($data[0]['tracking_number']);
        }
        return $response;
    }

    
    public function getTrackingInfo($trackingNumber)
    {
        $tracking = $this->_trackStatusFactory->create();
        $url = 'https://www.chilexpress.cl/Views/ChilexpressCL/Resultado-busqueda.aspx?DATA=' . $trackingNumber; // this is the tracking URL of stamps.com, replace this with your's
        $tracking->setData([
            'carrier' => $this->_code,
            'carrier_title' => $this->getConfigData('title'),
            'tracking' => $trackingNumber,
            'url' => $url
        ]);
        return $tracking;
    }

    
}
